@@ -1,3 +1,17 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2009-2013 Edgewall Software
+# Copyright (C) 2009 Remy Blank <remy.blank@pobox.com>
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/.
+
 import re
 
 from trac.config import Option
@@ -6,8 +20,8 @@ from trac.resource import ResourceNotFound
 from trac.ticket.api import IMilestoneChangeListener
 from trac.ticket.model import Version
 
-revision = "$Rev: 11490 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/tags/trac-1.0.1/sample-plugins/milestone_to_version.py $"
+revision = "$Rev: 13230 $"
+url = "$URL: http://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/milestone_to_version.py $"
 
 
 class MilestoneToVersion(Component):
@@ -46,19 +60,19 @@ class MilestoneToVersion(Component):
                 version.time = milestone.completed
                 version.update()
                 self.log.info('Existing version "%s" updated with completion '
-                              'time from milestone "%s"' %
-                              (version.name, milestone.name))
+                              'time from milestone "%s"', version.name,
+                              milestone.name)
             else:
                 self.log.info('Version "%s" already exists.  No new version '
-                              'created from milestone "%s"' %
-                              (version.name, milestone.name))
+                              'created from milestone "%s"', version.name,
+                              milestone.name)
         except ResourceNotFound:
             version = Version(self.env)
             version.name = version_name
             version.time = milestone.completed
             version.insert()
             self.log.info('New version "%s" created from completed milstone '
-                          '"%s".' % (version.name, milestone.name))
+                          '"%s".', version.name, milestone.name)
 
     def milestone_deleted(self, milestone):
         pass
