@@ -38,26 +38,25 @@ from trac.ticket.api import TicketSystem, ITicketManipulator
 from trac.ticket.model import Milestone, Ticket, group_milestones
 from trac.ticket.notification import TicketNotifyEmail
 from trac.timeline.api import ITimelineEventProvider
-from trac.util import as_bool, as_int, get_reporter_id
+from trac.util import as_bool, as_int, get_reporter_id, lazy
 from trac.util.datefmt import (
     format_datetime, from_utimestamp, to_utimestamp, utc
 )
+from trac.util.html import to_fragment
 from trac.util.text import (
-    exception_to_unicode, empty, obfuscate_email_address, shorten_line,
-    to_unicode
+    exception_to_unicode, empty, obfuscate_email_address, shorten_line
 )
 from trac.util.presentation import separated
-from trac.util.translation import _, tag_, tagn_, N_, gettext, ngettext
+from trac.util.translation import _, tag_, tagn_, N_, ngettext
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
-from trac.web import (
-    IRequestHandler, RequestDone, arg_list_to_args, parse_arg_list
-)
+from trac.web.api import IRequestHandler, arg_list_to_args, parse_arg_list
 from trac.web.chrome import (
     Chrome, INavigationContributor, ITemplateProvider,
     add_ctxtnav, add_link, add_notice, add_script, add_script_data,
-    add_stylesheet, add_warning, auth_link, prevnext_nav, web_context
+    add_stylesheet, add_warning, auth_link, chrome_info_script, prevnext_nav,
+    web_context
 )
-from trac.wiki.formatter import format_to, format_to_html, format_to_oneliner
+from trac.wiki.formatter import format_to, format_to_html
 
 
 class InvalidTicket(TracError):
@@ -77,12 +76,14 @@ class TicketModule(Component):
         open / close operations (''since 0.9'').""")
 
     max_description_size = IntOption('ticket', 'max_description_size', 262144,
-        """Don't accept tickets with a too big description.
-        (''since 0.11'').""")
+        """Maximum allowed description size in characters.
+        (//since 0.11//).""")
 
     max_comment_size = IntOption('ticket', 'max_comment_size', 262144,
-        """Don't accept tickets with a too big comment.
-        (''since 0.11.2'')""")
+        """Maximum allowed comment size in characters. (//since 0.11.2//).""")
+
+    max_summary_size = IntOption('ticket', 'max_summary_size', 262144,
+        """Maximum allowed summary size in characters. (//since 1.0.2//).""")
 
     timeline_newticket_formatter = Option('timeline', 'newticket_formatter',
                                           'oneliner',
@@ -106,6 +107,8 @@ class TicketModule(Component):
             [TracQuery#UsingTracLinks Trac links].
             (''since 0.12'')""")
 
+    ticket_path_re = re.compile(r'/ticket/([0-9]+)$')
+
     def __init__(self):
         self._warn_for_default_attr = set()
 
@@ -123,11 +126,11 @@ class TicketModule(Component):
             return getattr(TicketSystem(self.env), name)
         raise AttributeError("TicketModule has no attribute '%s'" % name)
 
-    @property
+    @lazy
     def must_preserve_newlines(self):
         preserve_newlines = self.preserve_newlines
         if preserve_newlines == 'default':
-            preserve_newlines = self.env.get_version(initial=True) >= 21 # 0.11
+            preserve_newlines = self.env.database_initial_version >= 21 # 0.11
         return as_bool(preserve_newlines)
 
     # IContentConverter methods
@@ -147,14 +150,14 @@ class TicketModule(Component):
             return self.export_csv(req, ticket, sep='\t',
                                    mimetype='text/tab-separated-values')
         elif key == 'rss':
-            return self.export_rss(req, ticket)
+            return self._export_rss(req, ticket)
 
     # INavigationContributor methods
 
     def get_active_navigation_item(self, req):
-        if re.match(r'/newticket/?', req.path_info):
-            return 'newticket'
-        return 'tickets'
+        if self.ticket_path_re.match(req.path_info):
+            return 'tickets'
+        return 'newticket'
 
     def get_navigation_items(self, req):
         if 'TICKET_CREATE' in req.perm:
@@ -165,12 +168,12 @@ class TicketModule(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        if req.path_info == "/newticket":
-            return True
-        match = re.match(r'/ticket/([0-9]+)$', req.path_info)
+        match = self.ticket_path_re.match(req.path_info)
         if match:
             req.args['id'] = match.group(1)
             return True
+        if req.path_info == '/newticket':
+            return True
 
     def process_request(self, req):
         if 'id' in req.args:
@@ -296,7 +299,7 @@ class TicketModule(Component):
                     FROM ticket_change tc
                         INNER JOIN ticket t ON t.id = tc.ticket
                             AND tc.time>=%s AND tc.time<=%s
-                    ORDER BY tc.time
+                    ORDER BY tc.time, tc.ticket
                     """ % (ts_start, ts_stop)):
                 if not (oldvalue or newvalue):
                     # ignore empty change corresponding to custom field
@@ -307,7 +310,7 @@ class TicketModule(Component):
                         ev = produce_event(data, status, fields, comment,
                                            cid)
                         if ev:
-                             yield (ev, data[1])
+                            yield (ev, data[1])
                     status, fields, comment, cid = 'edit', {}, '', None
                     data = (id, t, author, type, summary, None)
                 if field == 'comment':
@@ -440,7 +443,7 @@ class TicketModule(Component):
                 yield controller
 
     def _process_newticket_request(self, req):
-        req.perm.require('TICKET_CREATE')
+        req.perm('ticket').require('TICKET_CREATE')
         ticket = Ticket(self.env)
 
         plain_fields = True # support for /newticket?version=0.11 GETs
@@ -503,6 +506,7 @@ class TicketModule(Component):
 
         if req.get_header('X-Requested-With') == 'XMLHttpRequest':
             data['preview_mode'] = True
+            data['chrome_info_script'] = chrome_info_script
             return 'ticket_box.html', data, None
 
         add_stylesheet(req, 'common/css/ticket.css')
@@ -521,7 +525,8 @@ class TicketModule(Component):
             escape_newlines = self.must_preserve_newlines
             rendered = format_to_html(self.env, context,
                                       req.args.get('edited_comment', ''),
-                                      escape_newlines=escape_newlines)
+                                      escape_newlines=escape_newlines) + \
+                       chrome_info_script(req)
             req.send(rendered.encode('utf-8'))
 
         req.perm('ticket', id, version).require('TICKET_VIEW')
@@ -643,6 +648,7 @@ class TicketModule(Component):
 
         if xhr:
             data['preview_mode'] = bool(data['change_preview']['fields'])
+            data['chrome_info_script'] = chrome_info_script
             return 'ticket_preview.html', data, None
 
         mime = Mimeview(self.env)
@@ -754,7 +760,7 @@ class TicketModule(Component):
                 remove.append(entry)
             else:
                 add.append(entry)
-        action = entry = ''
+        action = entry = None
         if remove:
             action, entry = ('remove', remove[0])
         elif add:
@@ -1130,7 +1136,7 @@ class TicketModule(Component):
         for f in fields:
             name = f['name']
             value = ticket.values.get(name, '')
-            if name in ('cc', 'reporter'):
+            if name in ('cc', 'owner', 'reporter'):
                 value = Chrome(self.env).format_emails(context, value, ' ')
             elif name in ticket.time_fields:
                 value = format_datetime(value, '%Y-%m-%d %H:%M:%S',
@@ -1140,6 +1146,13 @@ class TicketModule(Component):
         return (content.getvalue(), '%s;charset=utf-8' % mimetype)
 
     def export_rss(self, req, ticket):
+        """:deprecated: since 1.0.6, use `_export_rss` instead. Will be
+                        removed in 1.3.1.
+        """
+        content, content_type = self._export_rss(req, ticket)
+        return ''.join(content), content_type
+
+    def _export_rss(self, req, ticket):
         changes = []
         change_summary = {}
 
@@ -1177,7 +1190,8 @@ class TicketModule(Component):
         data = self._prepare_data(req, ticket, absurls=True)
         data['changes'] = changes
         output = Chrome(self.env).render_template(req, 'ticket.rss', data,
-                                                  'application/rss+xml')
+                                                  'application/rss+xml',
+                                                  iterable=True)
         return output, 'application/rss+xml'
 
     # Ticket validation and changes
@@ -1241,8 +1255,9 @@ class TicketModule(Component):
                 value = ticket[name]
                 if value:
                     if value not in field['options']:
-                        add_warning(req, '"%s" is not a valid value for '
-                                    'the %s field.' % (value, name))
+                        add_warning(req, _('"%(value)s" is not a valid value '
+                                           'for the %(name)s field.',
+                                           value=value, name=name))
                         valid = False
                 elif not field.get('optional', False):
                     add_warning(req, _("field %(name)s must be set",
@@ -1263,6 +1278,13 @@ class TicketModule(Component):
                                num=self.max_comment_size))
             valid = False
 
+        # Validate summary length
+        if len(ticket['summary'] or '') > self.max_summary_size:
+            add_warning(req, _("Ticket summary is too long (must be less "
+                               "than %(num)s characters)",
+                               num=self.max_summary_size))
+            valid = False
+
         # Validate comment numbering
         try:
             # replyto must be 'description' or a number
@@ -1278,9 +1300,9 @@ class TicketModule(Component):
             for field, message in manipulator.validate_ticket(req, ticket):
                 valid = False
                 if field:
-                    add_warning(req, _("The ticket field '%(field)s' is "
-                                       "invalid: %(message)s",
-                                       field=field, message=message))
+                    add_warning(req, tag_("The ticket field '%(field)s'"
+                                          " is invalid: %(message)s",
+                                          field=field, message=message))
                 else:
                     add_warning(req, message)
         return valid
@@ -1289,15 +1311,15 @@ class TicketModule(Component):
         ticket.insert()
 
         # Notify
+        tn = TicketNotifyEmail(self.env)
         try:
-            tn = TicketNotifyEmail(self.env)
             tn.notify(ticket, newticket=True)
         except Exception, e:
             self.log.error("Failure sending notification on creation of "
                     "ticket #%s: %s", ticket.id, exception_to_unicode(e))
-            add_warning(req, _("The ticket has been created, but an error "
-                               "occurred while sending notifications: "
-                               "%(message)s", message=to_unicode(e)))
+            add_warning(req, tag_("The ticket has been created, but an error "
+                                  "occurred while sending notifications: "
+                                  "%(message)s", message=to_fragment(e)))
 
         # Redirect the user to the newly created ticket or add attachment
         ticketref=tag.a('#', ticket.id, href=req.href.ticket(ticket.id))
@@ -1328,8 +1350,8 @@ class TicketModule(Component):
                                    replyto=req.args.get('replyto'))
         if cnum:
             fragment = '#comment:%d' % cnum
+            tn = TicketNotifyEmail(self.env)
             try:
-                tn = TicketNotifyEmail(self.env)
                 tn.notify(ticket, newticket=False, modtime=now)
             except Exception, e:
                 self.log.error("Failure sending notification on change to "
@@ -1341,12 +1363,12 @@ class TicketModule(Component):
                 add_warning(req, tag_("The %(change)s has been saved, but an "
                                       "error occurred while sending "
                                       "notifications: %(message)s",
-                                      change=change, message=to_unicode(e)))
+                                      change=change, message=to_fragment(e)))
                 fragment = ''
 
         # After saving the changes, apply the side-effects.
         for controller in controllers:
-            self.log.debug("Side effect for %s" %
+            self.log.debug("Side effect for %s",
                            controller.__class__.__name__)
             controller.apply_action_side_effects(req, ticket, action)
 
@@ -1401,6 +1423,9 @@ class TicketModule(Component):
 
     def _query_link(self, req, name, value, text=None):
         """Return a link to /query with the appropriate name and value"""
+        from trac.ticket.query import QueryModule
+        if not self.env.is_component_enabled(QueryModule):
+            return text or value
         default_query = self.ticketlink_query.lstrip('?')
         args = arg_list_to_args(parse_arg_list(default_query))
         args[name] = value
@@ -1410,7 +1435,9 @@ class TicketModule(Component):
 
     def _query_link_words(self, context, name, value):
         """Splits a list of words and makes a query link to each separately"""
-        if not isinstance(value, basestring): # None or other non-splitable
+        from trac.ticket.query import QueryModule
+        if not (isinstance(value, basestring) and  # None or other non-splitable
+                self.env.is_component_enabled(QueryModule)):
             return value
         default_query = self.ticketlink_query.startswith('?') and \
                         self.ticketlink_query[1:] or self.ticketlink_query
@@ -1439,6 +1466,12 @@ class TicketModule(Component):
             name = field['name']
             type_ = field['type']
 
+            # ensure sane defaults
+            field.setdefault('optional', False)
+            field.setdefault('options', [])
+            field.setdefault('skip', False)
+            field.setdefault('editable', True)
+
             # enable a link to custom query for all choice fields
             if type_ not in ['text', 'textarea']:
                 field['rendered'] = self._query_link(req, name, ticket[name])
@@ -1461,6 +1494,7 @@ class TicketModule(Component):
                               for opt in field['options']]
                 milestones = [m for m in milestones
                               if 'MILESTONE_VIEW' in req.perm(m.resource)]
+                field['editable'] = milestones != []
                 groups = group_milestones(milestones, ticket.exists
                     and 'TICKET_ADMIN' in req.perm(ticket.resource))
                 field['options'] = []
@@ -1481,9 +1515,10 @@ class TicketModule(Component):
                     field['edit_label'] = {
                             'add': _("Add to Cc"),
                             'remove': _("Remove from Cc"),
-                            '': _("Add/Remove from Cc")}[cc_action]
-                    field['cc_entry'] = cc_entry or _("<Author field>")
-                    field['cc_update'] = cc_update or None
+                            None: _("Cc")}[cc_action]
+                    field['cc_action'] = cc_action
+                    field['cc_entry'] = cc_entry
+                    field['cc_update'] = cc_update
                     if cc_changed:
                         field_changes['cc']['cc_update'] = cc_update
                 if cc_changed:
@@ -1499,7 +1534,7 @@ class TicketModule(Component):
 
             # per type settings
             if type_ in ('radio', 'select'):
-                if ticket.exists:
+                if ticket.exists and field['editable']:
                     value = ticket.values.get(name)
                     options = field['options']
                     optgroups = []
@@ -1517,25 +1552,13 @@ class TicketModule(Component):
                     field['rendered'] = self._query_link(req, name, value,
                                 _("yes") if value == '1' else _("no"))
             elif type_ == 'text':
-                if field.get('format') == 'wiki':
-                    field['rendered'] = format_to_oneliner(self.env, context,
-                                                           ticket[name])
-                elif field.get('format') == 'reference':
+                if field.get('format') == 'reference':
                     field['rendered'] = self._query_link(req, name,
                                                          ticket[name])
                 elif field.get('format') == 'list':
                     field['rendered'] = self._query_link_words(context, name,
                                                                ticket[name])
-            elif type_ == 'textarea':
-                if field.get('format') == 'wiki':
-                    field['rendered'] = \
-                        format_to_html(self.env, context, ticket[name],
-                                escape_newlines=self.must_preserve_newlines)
 
-            # ensure sane defaults
-            field.setdefault('optional', False)
-            field.setdefault('options', [])
-            field.setdefault('skip', False)
             fields.append(field)
 
         # Move owner field to end when shown
